/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.communication.jndi.agent;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.NamingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.Contingency;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.jndi.agent.msg.request.RegistryRequest;
import org.ow2.petals.communication.jndi.agent.util.RegistryUtil;
import org.ow2.petals.communication.network.NetworkService;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.DomainConfiguration;
import org.ow2.petals.util.LoggingUtil;

/**
 * The implementation of the PEtALS JNDI agent {@link JNDIAgentService}.
 *
 * @author alouis, Roland NAUDIN - EbmWebsourcing
 * @since Petals 2.0
 *
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.communication.jndi.agent.JNDIAgentService.class))
public class JNDIAgentServiceImpl implements JNDIAgentService {

    /**
     * Logging utilities
     */
    protected LoggingUtil log;

    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * The Network service fractal component
     */
    @Requires(name = "network", signature = org.ow2.petals.communication.network.NetworkService.class, contingency = Contingency.OPTIONAL)
    private NetworkService networkService;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The domain configuration
     */
    private DomainConfiguration domainConfiguration;

    /**
     * Data stored in this distributed registry
     */
    protected Map<String, HashMap<String, Object>> data;

    /**
     * Registry utility
     */
    protected RegistryUtil registryUtil;


    /**
     * Process a JNDI request
     *
     * @param request
     *            request to process
     * @return output Object of the operation
     * @throws NamingException
     */
    public Object processRequest(RegistryRequest request) throws NamingException {
        Object output = null;
        switch (request.getType()) {
            /*
             * BIND
             */
            case bind:
                registryUtil.bind(request.getArg1(), request.getArg2(), request.getArg3());
                break;
            /*
             * CREATE SUBCONTEXT
             */
            case createSubcontext:
                output = registryUtil.createSubcontext(request.getArg1(), request.getArg2());
                break;
            /*
             * DESTROY SUBCONTEXT
             */
            case destroySubcontext:
                registryUtil.destroySubcontext(request.getArg1(), request.getArg2());
                break;
            /*
             * LIST
             */
            case list:
                output = registryUtil.list(request.getArg1(), request.getArg2());
                break;
            /*
             * LIST BINDINGS
             */
            case listBindings:
                output = registryUtil.listBindings(request.getArg1(), request.getArg2(), request
                        .isLocalConnection());
                break;
            /*
             * LOOKUP
             */
            case lookup:
                output = registryUtil.lookup(request.getArg1(), request.getArg2(), request
                        .isLocalConnection());
                break;
            /*
             * LOOKUP LINK
             */
            case lookupLink:
                output = registryUtil.lookupLink(request.getArg1(), request.getArg2(), request
                        .isLocalConnection());
                break;
            /*
             * REBIND
             */
            case rebind:
                registryUtil.rebind(request.getArg1(), request.getArg2(), request.getArg3());
                break;
            /*
             * RENAME
             */
            case rename:
                registryUtil.rename(request.getArg1(), request.getArg2(), request.getArg3(),
                        request.isLocalConnection());
                break;
            /*
             * UNBIND
             */
            case unbind:
                registryUtil.unbind(request.getArg1(), request.getArg2());
                break;

            default:
                break;
        }
        return output;
    }

    public LoggingUtil getLog() {
        return log;
    }

    public DomainConfiguration getDomainConfiguration() {
        return domainConfiguration;
    }

    public Map<String, HashMap<String, Object>> getData() {
        return data;
    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws JNDIAgentException {
        log = new LoggingUtil(logger);
        log.start();

        domainConfiguration = configurationService.getDomainConfiguration();

        // start the Petals JNDI agent if necessary
        // (no external JNDI server is referenced)
        if (domainConfiguration.getJndiFactory()== null) {
                initData();
                registryUtil = new RegistryUtil(this);
        }

        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        log.start();
        log.end();
    }

    /**
     * Initialize the JNDI data
     */
    private void initData() {
    	log.start();
        //The standalone distribution doesn't provide the network service
        if (networkService == null) {
            data = new Hashtable<String, HashMap<String, Object>>();
        } else {
            data = networkService.getReplicatedHashMap();
        }

        // init the root context if this is the first node
        if (!data.containsKey("/")) {
            HashMap<String, Object> newMap = new HashMap<String, Object>();
            data.put("/", newMap);

        }
        log.end();
    }


}
