/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AutoLoaderServiceImpl.java,v 1.2 2006/04/07 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.autoload;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.apache.commons.io.FileUtils;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.deployment.DeploymentServiceMBean;
import org.ow2.petals.jbi.management.installation.InstallationServiceMBean;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.jbi.management.util.XMLResult;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.api.server.util.SystemUtil;
import org.ow2.petals.kernel.server.PetalsServerImpl;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.systemstate.ComponentState;
import org.ow2.petals.platform.systemstate.ServiceAssemblyState;
import org.ow2.petals.platform.systemstate.SharedLibraryState;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This class is used to load automatically components which are placed in the
 * "autoLoad" directory. It scans the directory periodically and check if new
 * components are presents. The {@ link InstallationServiceMBean} is called for
 * each of those components. The whole process of installation is done (install +
 * start) After being threat, the archives of the components are removed. A file
 * "fileName.success" will be created if success A file "fileName.failed" will
 * be created otherwise, containing the related exception
 * 
 * @author alouis
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.management.autoload.AutoLoaderService.class))
public class AutoLoaderServiceImpl implements AutoLoaderService {

    private static final String install = "install";

    private static final String installed = "installed";

    private static final String uninstalled = "uninstalled";

    private static final long scanPeriod = 4000L;

    @Monolog(name = "logger")
    protected Logger logger;

    protected LoggingUtil log;

    protected static File installDirectory;

    protected static File installedDirectory;

    protected static File uninstalledDirectory;

    protected static File workDirectory;

    protected Timer installDirectoryScanTimer;

    protected Timer installedDirectoryScanTimer;

    /**
     * Package Handler
     */
    protected PackageHelper packageHelper;

    @Requires(name = "deployment", signature = org.ow2.petals.jbi.management.deployment.DeploymentServiceMBean.class)
    protected DeploymentServiceMBean deploymentService;

    @Requires(name = "installation", signature = org.ow2.petals.jbi.management.installation.InstallationServiceMBean.class)
    protected InstallationServiceMBean installationService;

    @Requires(name = "admin", signature = org.ow2.petals.jbi.management.admin.AdminServiceMBean.class)
    protected AdminServiceMBean adminService;

    /**
     * Container :: System state service
     */
    @Requires(name = "systemstate", signature = org.ow2.petals.platform.systemstate.SystemStateService.class)
    protected SystemStateService systemStateService;

    /**
     * Platform Component :: Repository Service
     */
    @Requires(name = "repository", signature = org.ow2.petals.platform.repository.RepositoryService.class)
    protected RepositoryService repositorySrv;

    public File getInstalledDirectory() {
        return installedDirectory;
    }

    public File getUninstalledDirectory() {
        return uninstalledDirectory;
    }

    public File getWorkDirectory() {
        return workDirectory;
    }

    /**
     * init the autoloader : init the directories
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IOException {
        log = new LoggingUtil(logger);
        log.call();
        installDirectory = new File(SystemUtil.getPetalsInstallDirectory().getAbsolutePath()
                + File.separator + install);
        installedDirectory = new File(SystemUtil.getPetalsInstallDirectory().getAbsolutePath()
                + File.separator + installed);
        uninstalledDirectory = new File(SystemUtil.getPetalsInstallDirectory().getAbsolutePath()
                + File.separator + uninstalled);
        workDirectory = new File(SystemUtil.getPetalsInstallDirectory().getAbsolutePath()
                + File.separator + PetalsServerImpl.WORK_DIRECTORY);

        checkValidityOfDirectory(installDirectory);
        checkValidityOfDirectory(installedDirectory);
        checkValidityOfDirectory(uninstalledDirectory);

        // old start method
        installDirectoryScanTimer = new Timer();
        installedDirectoryScanTimer = new Timer();
        InstallDirectoryScanner installScan = new InstallDirectoryScanner(this, installDirectory);
        InstalledDirectoryScanner installedScan = new InstalledDirectoryScanner(this,
                installedDirectory);
        /*
         * Initialize package handler
         */
        packageHelper = new PackageHelper(logger, repositorySrv);

        installDirectoryScanTimer.schedule(installScan, scanPeriod, scanPeriod);
        installedDirectoryScanTimer.schedule(installedScan, scanPeriod, scanPeriod);
    }

    /**
     * Stop the autoload of the components. TODO : synchronization, does the
     * autoload must wait before the end of an installation before closing?
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        log.call();

        if (installDirectoryScanTimer != null) {
            installDirectoryScanTimer.cancel();
        }
        if (installedDirectoryScanTimer != null) {
            installedDirectoryScanTimer.cancel();
        }
    }

    /**
     * @see org.ow2.petals.jbi.management.autoload.AutoLoaderService#install(java.io.File[])
     */
    public void install(File[] newInstallers) {
        log.call();

        if (newInstallers != null) {
            for (int i = 0; i < newInstallers.length; i++) {
                File file = newInstallers[i];

                File workArchive = null;
                try {
                    /*
                     * Copy archive file to work directory
                     */
                    workArchive = new File(getWorkDirectory(), file.getName());
                    FileUtils.copyFile(file, workArchive);
                    FileUtils.forceDelete(file);

                    /*
                     * Load JBI descriptor here
                     */
                    Jbi descriptor = packageHelper.loadDescriptor(workArchive.toURI());
                    /*
                     * Perform the right task : install, share or deploy
                     */
                    if (descriptor.getComponent() != null) {
                        performInstallComponent(workArchive);
                    } else if (descriptor.getSharedLibrary() != null) {
                        performInstallSL(workArchive);
                    } else if (descriptor.getServiceAssembly() != null) {
                        performDeploySA(workArchive);
                    } else {
                        throw new PetalsException("Failed to identify the JBI artifact : "
                                + workArchive.getCanonicalPath());
                    }

                } catch (Throwable e) {
                    log.error("Error during installation of JBI artifact", e);
                } finally {
                    if (workArchive != null) {
                        try {
                            FileUtils.forceDelete(workArchive);
                        } catch (IOException e) {
                            log.error("Failed to delete a temporary file: "
                                    + workArchive.getAbsolutePath(), e);
                        }
                        workArchive = null;
                    }
                }

            }
        }
    }

    /**
     * @see org.ow2.petals.jbi.management.autoload.AutoLoaderService#uninstall(java.util.List)
     */
    public void uninstall(List<File> filesToUninstall) {
        log.call();
        /*
         * Retrieve all installed entities
         */
        List<ComponentState> compStates = systemStateService.retrieveAllComponentStates();
        List<SharedLibraryState> slStates = systemStateService.retrieveAllSharedLibraryStates();
        List<ServiceAssemblyState> saStates = systemStateService.retrieveAllServiceAssemblyStates();

        try {
            /*
             * Check if one of the files to uninstall match one of the
             * archiveUrl of the installed entities. If yes, this entity must be
             * uninstalled.
             */
            List<String> compToUninstall = new ArrayList<String>();
            List<String> slToUninstall = new ArrayList<String>();
            List<String> saToUninstall = new ArrayList<String>();
            for (ComponentState state : compStates) {
                URI archiveURI = packageHelper.getPackageURI(state.getArchiveURL(), false);
                File archiveFile = new File(archiveURI);
                for (File file : filesToUninstall) {
                    if (file.equals(archiveFile)) {
                        compToUninstall.add(state.getName());
                    }
                }
            }
            for (SharedLibraryState state : slStates) {
                URI archiveURI = packageHelper.getPackageURI(state.getArchiveURL(), false);
                File archiveFile = new File(archiveURI);
                for (File file : filesToUninstall) {
                    if (file.equals(archiveFile)) {
                        slToUninstall.add(state.getName());
                    }
                }
            }
            for (ServiceAssemblyState state : saStates) {
                URI archiveURI = packageHelper.getPackageURI(state.getArchiveURL(), false);
                File archiveFile = new File(archiveURI);
                for (File file : filesToUninstall) {
                    if (file.equals(archiveFile)) {
                        saToUninstall.add(state.getName());
                    }
                }
            }

            /*
             * Perform entity uninstallation
             */
            for (String name : saToUninstall) {
                performUndeploySA(name);
            }
            for (String name : compToUninstall) {
                performUninstallComponent(name);
            }
            for (String name : slToUninstall) {
                performUninstallSL(name);
            }
        } catch (Throwable e) {
            log.error("Error during uninstallation of JBI artifact(s)", e);
        }

    }

    /**
     * Install the specified Archive in the JBI container and start it . Any
     * exception will cause the abortion of the process. A file
     * "fileName.success" will be created if success A file "fileName.failed"
     * will be created otherwise, containing the related exception
     * 
     * @param f
     */
    protected void performInstallComponent(File componentFile) {
        log.call();

        if (componentFile != null && componentFile.exists()) {

            try {
                MBeanServer jmxServer = adminService.getMBeanServer();

                ObjectName newInstallerName = installationService.loadNewInstaller(componentFile
                        .toURI().toString());
                if (!newInstallerName.getDomain().equals("error")) {
                    Object[] nullObjects = new Object[0];
                    String[] nullStrings = new String[0];

                    Object result = jmxServer.invoke(newInstallerName, "install", nullObjects,
                            nullStrings);

                    if (result instanceof ObjectName) {
                        ObjectName componentName = (ObjectName) result;

                        jmxServer.invoke(componentName, "start", nullObjects, nullStrings);

                    } else {
                        throw new Exception(
                                "The install() invokation on the component installer has returned an unexpected object: "
                                        + result);
                    }
                } else {
                    throw new Exception("Component can't be auto installed");
                }
            } catch (Throwable e) {
                log.error("Error occured during auto installation", e);
                createFailureFile(componentFile, e);
            }
        }
    }

    /**
     * Uninstall the specified component.
     * 
     * @param name
     */
    protected void performUninstallComponent(String name) {
        installationService.forceUnloadComponentInstaller(name);
    }

    /**
     * Deploy the specified Archive in the JBI container and start it . Any
     * exception will cause the abortion of the process. A file
     * "fileName.success" will be created if success A file "fileName.failed"
     * will be created otherwise, containing the related exception
     * 
     * @param saFile
     *            the archive file
     */
    protected void performDeploySA(File saFile) {
        log.call();

        if (saFile != null && saFile.exists()) {

            try {
                String xml = deploymentService.deploy(saFile.toURI().toString());

                if (xml.indexOf(XMLResult.TaskResult.FAILED.toString()) > -1) {
                    log.warning("Problem while deploying a service assembly\n" + xml);
                } else {
                    String saName = "";
                    saName = xml.substring(xml.indexOf("<loc-param>") + 11, xml
                            .indexOf("</loc-param>"));
                    deploymentService.start(saName);

                }
            } catch (Throwable e) {
                log.error("Error occured during auto deployment", e);
                createFailureFile(saFile, e);
            }
        }
    }

    /**
     * Undeploy the specified service assembly. isToBeDeleted is setted to true !
     * 
     * @param name
     */
    protected void performUndeploySA(String name) {
        deploymentService.forceUndeploy(name);
    }

    /**
     * Install the specified shared library in the JBI container Any exception
     * will cause the abortion of the process. A file "fileName.success" will be
     * created if success A file "fileName.failed" will be created otherwise,
     * containing the related exception
     * 
     * @param f
     */
    protected void performInstallSL(File slFile) {
        log.call();

        if (slFile != null && slFile.exists()) {

            try {
                installationService.installSharedLibrary(slFile.toURI().toString());

            } catch (Throwable e) {
                createFailureFile(slFile, e);
                log.error("Error occured during auto Shared library installation.", e);
            }
        }
    }

    /**
     * Uninstall the specified shared lib.
     * 
     * @param name
     */
    protected void performUninstallSL(String name) {
        installationService.uninstallSharedLibrary(name);
    }

    /**
     * Check if the specified File is a directory and is created. If it does not
     * exist, create it.
     * 
     * @param dir
     * @param clean
     *            delete directory content or not
     */
    private void checkValidityOfDirectory(File dir) throws IOException {
        if (dir != null) {
            if (!dir.exists()) {
                if (!dir.mkdirs()) {
                    throw new IOException("can not create the following directory : "
                            + dir.getCanonicalPath());
                }
            } else {
                if (!dir.isDirectory()) {
                    throw new IOException(dir.getCanonicalPath()
                            + " already exists and is not a directory.");
                }
            }
        } else {
            throw new FileNotFoundException("the specified directory is null");
        }
    }

    /**
     * Create an error report in the work directory.
     * 
     * @param file
     * @param exception
     * @throws IOException
     * @throws FileNotFoundException
     */
    private void createFailureFile(File file, Throwable exception) {
        File failed = new File(workDirectory, file.getName() + ".failed");

        try {
            failed.createNewFile();
            PrintWriter w = new PrintWriter(failed);
            exception.printStackTrace(w);
            w.flush();
            w.close();
        } catch (FileNotFoundException e) {
            log.error("Error occured during failure report creation.", e);
        } catch (IOException e) {
            log.error("Error occured during failure report creation.", e);
        }

    }

}
