/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: CreateAndRegisterSULifeCycleTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.net.URI;
import java.util.HashMap;

import javax.jbi.JBIException;

import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.processor.Task;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task creates a service unit lifecycle and registers this lifecycle into
 * the sa lifecycle <br/> Inputs : {@link Jbi} of the SU,
 * {@link ServiceAssemblyLifeCycle}, SU install root {@link URI} <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class CreateAndRegisterSULifeCycleTask implements Task {

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminServiceMBean adminService;

    public CreateAndRegisterSULifeCycleTask(LoggingUtil log, AdminServiceMBean adminService) {
        super();
        this.log = log;
        this.adminService = adminService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        URI installationRoot = (URI) context.get(DeploymentContextConstants.SU_INSTALL_ROOT);
        ServiceUnit serviceUnit = (ServiceUnit) context
                .get(DeploymentContextConstants.SU_DESCRIPTOR);
        ServiceAssemblyLifeCycle saLifeCycle = (ServiceAssemblyLifeCycle) context
                .get(DeploymentContextConstants.SA_LIFECYCLE);

        // Create SU lifecycle
        ServiceUnitLifeCycle suLifeCycle = createNewServiceUnitLifeCycle(
                installationRoot.getPath(), serviceUnit);

        // Register this SU lifecycle into the SA lifecycle
        registerSULifeCycle(suLifeCycle, saLifeCycle, serviceUnit);

    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws ManagementException {
        log.debug("Revert CreateAndRegisterSULifeCycleTask");
        ServiceUnit serviceUnit = (ServiceUnit) context
                .get(DeploymentContextConstants.SU_DESCRIPTOR);

        ServiceAssemblyLifeCycle saLifeCycle = (ServiceAssemblyLifeCycle) context
                .get(DeploymentContextConstants.SA_LIFECYCLE);

        unregisterSULifeCycle(saLifeCycle, serviceUnit);
    }

    /**
     * Creates a new {@link ServiceUnitLifeCycle}
     * 
     * @param rootPath
     *            the SU installation root
     * @param serviceUnit
     *            the service unit production element
     * @return a new {@link ServiceUnitLifeCycle} instance
     */
    protected ServiceUnitLifeCycle createNewServiceUnitLifeCycle(String rootPath,
            ServiceUnit serviceUnit) {
        return new ServiceUnitLifeCycle(serviceUnit, adminService, rootPath, log);
    }

    /**
     * Register the {@link ServiceUnitLifeCycle} to the
     * {@link ServiceAssemblyLifeCycle}.
     * 
     * @param suLifeCycle
     *            the {@link ServiceUnitLifeCycle} to register
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be registered
     * @param serviceUnit
     *            the service unit production element
     * @throws ManagementException
     *             if it fails to register {@link ServiceUnitLifeCycle} to
     *             {@link ServiceAssemblyLifeCycle}
     */
    protected void registerSULifeCycle(ServiceUnitLifeCycle suLifeCycle,
            ServiceAssemblyLifeCycle saLifeCycle, ServiceUnit serviceUnit)
            throws ManagementException {
        String serviceUnitName = DeploymentUtils.getServiceUnitName(serviceUnit);
        try {
            saLifeCycle.registerSUIntoSA(serviceUnitName, suLifeCycle);
        } catch (JBIException e) {
            throw new ManagementException("Error while registering the service unit '"
                    + serviceUnitName + "' life cycle into the service assembly life cycle '"
                    + saLifeCycle.getServiceAssembly().getIdentification().getName() + "'", e);
        }
    }

    /**
     * Register the {@link ServiceUnitLifeCycle} to the
     * {@link ServiceAssemblyLifeCycle}.
     * 
     * @param saLifeCycle
     *            the {@link ServiceAssemblyLifeCycle} where
     *            {@link ServiceUnitLifeCycle} will be registered
     * @param serviceUnit
     *            the service unit production element of the su to unregister
     * @throws ManagementException
     *             if it fails to register {@link ServiceUnitLifeCycle} to
     *             {@link ServiceAssemblyLifeCycle}
     */
    protected void unregisterSULifeCycle(ServiceAssemblyLifeCycle saLifeCycle,
            ServiceUnit serviceUnit) throws ManagementException {
        saLifeCycle.unregisterSUFromSA(DeploymentUtils.getServiceUnitName(serviceUnit));

    }

}
