/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: RegisterAllConnectionTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.jbi.JBIException;
import javax.xml.namespace.QName;

import org.ow2.petals.jbi.descriptor.original.generated.Connection;
import org.ow2.petals.jbi.descriptor.original.generated.Connections;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.deployment.DeploymentUtils;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.processor.Task;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task registers connections if exist <br/> Inputs : {@link Jbi}
 * <br/>
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class RegisterAllConnectionTask implements Task {

    protected EndpointRegistry EndpointRegistry;

    /**
     * logger wrapper
     */
    protected LoggingUtil log;

    public RegisterAllConnectionTask(EndpointRegistry EndpointRegistry, LoggingUtil log) {
        super();
        this.EndpointRegistry = EndpointRegistry;
        this.log = log;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        Connections connections = descriptor.getServiceAssembly().getConnections();
        List<Connection> successfulyRegisteredConnections = new ArrayList<Connection>();

        if (connections != null) {
            for (Connection connection : connections.getConnection()) {
                try {
                    registerConnection(connection, descriptor);
                } catch (ManagementException e) {
                    // Unregister previously registered connection
                    for (Connection successConnection : successfulyRegisteredConnections) {
                        try {
                            removeConnection(successConnection);
                        } catch (Exception e1) {
                            log.error("Can't delete an already registered connection");
                        }
                    }
                    throw e;
                }
                successfulyRegisteredConnections.add(connection);
            }
        }
    }

    /**
     * Register a connection into the endpoint registry of the JBI container
     * 
     * @param connection
     *            The {@link Connection} to register
     * @param descriptor
     *            the service assembly descriptor
     * @throws ManagementException
     *             when container fails to register a connection into the
     *             endpoint registry
     */
    protected void registerConnection(Connection connection, Jbi descriptor)
            throws ManagementException {
        String saName = DeploymentUtils.getServiceAssemblyName(descriptor);

        QName consInterface = connection.getConsumer().getInterfaceName();
        QName consService = connection.getConsumer().getServiceName();
        String consEndpoint = connection.getConsumer().getEndpointName();
        QName provService = connection.getProvider().getServiceName();
        String provEndpoint = connection.getProvider().getEndpointName();

        try {
            if (consInterface != null) {

                EndpointRegistry.createConnection(consInterface, provService, provEndpoint);

            } else {
                EndpointRegistry.createConnection(consService, consEndpoint, provService,
                        provEndpoint);
            }
        } catch (JBIException e) {
            throw new ManagementException(
                    "Error while registering a service unit connection for the service assembly '"
                            + saName + "'", e);
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) {
        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        Connections connections = descriptor.getServiceAssembly().getConnections();
        if (connections != null) {
            for (Connection connection : connections.getConnection()) {
                try {
                    removeConnection(connection);
                } catch (Exception e) {
                    log.error("Failed to revert a RegisterAllConnectionTask", e);
                }
            }
        }
    }

    /**
     * Remove a connection registered during the SA deployment process
     * 
     * @param connection
     *            a Connection to remove
     * @throws Exception
     *             if problem occurs during the deletion of a specific
     *             connection
     */
    protected void removeConnection(Connection connection) throws Exception {
        QName consInterface = connection.getConsumer().getInterfaceName();
        QName consService = connection.getConsumer().getServiceName();
        String consEndpoint = connection.getConsumer().getEndpointName();
        QName provService = connection.getProvider().getServiceName();
        String provEndpoint = connection.getProvider().getEndpointName();

        if (provEndpoint != null && provService != null) {
            if (consInterface != null) {
                EndpointRegistry.deleteConnection(consInterface, provService, provEndpoint);
            } else if (consService != null && consEndpoint != null) {
                EndpointRegistry.deleteConnection(consService, consEndpoint, provService,
                        provEndpoint);
            }
        }
    }

}
