/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SACheckPackageTask.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.deploy;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.Connection;
import org.ow2.petals.jbi.descriptor.original.generated.Connections;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceAssembly;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.AbstractCheckPackageTask;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.StringHelper;

/**
 * This task checks the validity of the service assembly package. Some checking
 * steps are described in the specification (section 6.6 of the JSR 208).
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class SACheckPackageTask extends AbstractCheckPackageTask {

    protected SystemStateService systemStateService;

    protected AdminServiceMBean adminService;

    public SACheckPackageTask(LoggingUtil log, PackageHelper packageHandler,
            SystemStateService systemStateService, AdminServiceMBean adminService) {
        super(log, packageHandler);
        this.systemStateService = systemStateService;
        this.adminService = adminService;
    }

    @Override
    protected void checkPackage(Jbi descriptor, File filePackage) throws PetalsException {

        // Retrieve assembly production element into JBI descriptor
        ServiceAssembly assembly = descriptor.getServiceAssembly();
        if (assembly == null) {
            throw new PetalsException("Given archive isn't a service assembly archive : "
                    + "missing service assembly production element into jbi descriptor");
        }

        String saName = assembly.getIdentification().getName();

        // Check service assembly identifier
        if (StringHelper.isNullOrEmpty(saName)) {
            throw new PetalsException(
                    "Name of a service assembly must not be null or empty for package '"
                            + filePackage.getAbsolutePath() + "'");
        }

        // Check if SA name is already used
        if (isSADeployed(saName)) {
            throw new PetalsException("You are trying to deploy a sa with an identifier "
                    + "that is used by a previously successfuly deployed SA '" + saName + "'");
        }

        // Check potential connection elements
        Connections connections = assembly.getConnections();
        if (connections != null) {
            checkConnections(connections.getConnection());
        }

        // Check potential service unit elements
        List<ServiceUnit> sus = assembly.getServiceUnit();
        if (sus != null) {
            checkServiceUnits(sus, saName, filePackage);
        }
    }

    private boolean isSADeployed(String saName) {
        return adminService.getServiceAssemblyByName(saName) != null;
    }

    /**
     * Check the validity of connection elements
     * 
     * @param connections
     *            a {@link List} of {@link Connection} element
     * @throws Exception
     *             if consumer interface and consumer endpoint and consumer
     *             service are null or empty at the same time or if (provider
     *             endpoint and provider service) are null or empty
     */
    protected void checkConnections(List<Connection> connections) throws PetalsException {

        for (Connection connection : connections) {
            QName consInterface = connection.getConsumer().getInterfaceName();
            QName consService = connection.getConsumer().getServiceName();
            String consEndpoint = connection.getConsumer().getEndpointName();
            QName provService = connection.getProvider().getServiceName();
            String provEndpoint = connection.getProvider().getEndpointName();

            if (consInterface == null || StringHelper.isNullOrEmpty(consInterface.getLocalPart())) {
                if (StringHelper.isNullOrEmpty(consEndpoint) || consService == null
                        || StringHelper.isNullOrEmpty(consService.getLocalPart())) {
                    throw new PetalsException(
                            "Either Consumer interface or (consumer service and consumer endpoint) "
                                    + "must be filled in a Consumes connection");
                }
            }
            if (StringHelper.isNullOrEmpty(provEndpoint) || provService == null
                    || StringHelper.isNullOrEmpty(provService.getLocalPart())) {
                throw new PetalsException(
                        "Provider service and provider endpoint must be filled in a Provides connection");
            }
        }
    }

    /**
     * Check the validity of service unit elements
     * 
     * @param sus
     *            a {@link List} of {@link ServiceUnit} element
     * @param saName
     *            the service assembly name
     * @param filePackage
     *            the service assembly package root
     * @throws Exception
     *             if su identifier is null or empty or non unique within the
     *             sa, if target artifact zip and target component are null or
     *             empty, if not all declared su zip are presents in the sa
     *             package or if su has already been deployed to the target
     *             component
     */
    protected void checkServiceUnits(List<ServiceUnit> sus, String saName, File filePackage)
            throws PetalsException {

        List<String> suIds = new ArrayList<String>();
        for (ServiceUnit su : sus) {
            String suIdentifier = su.getIdentification().getName();

            // Check SU identifier : must be non null, non empty and unique
            // within the SA
            checkSUIdentifier(suIds, suIdentifier, saName);

            suIds.add(suIdentifier);

            // check target artifact zip and target component entries : must
            // be non null and non empty
            String artifactZip = su.getTarget().getArtifactsZip().trim();
            String targetComponent = su.getTarget().getComponentName();
            checkTargetComponentAndZip(artifactZip, suIdentifier, targetComponent);

            // check that all declared SU zip are presents in the SA package
            checkSUPackageInSAPackage(artifactZip, filePackage);

            // Check if SU hasn't already been deployed to the target
            // component
            if (isSUAlreadyDeployedOnTargetedComponent(su)) {
                throw new PetalsException("You are trying to deploy a SU '"
                        + su.getIdentification().getName()
                        + "' which have already been deployed on the target component '"
                        + su.getTarget().getComponentName() + "'");
            }

        }
    }

    /**
     * Check SU identifier : must be non null, non empty and unique within the
     * SA
     * 
     * @param suIds
     *            the SU identifier list within the SA
     * @param suIdentifier
     *            the service unit identifier
     * @param saName
     *            the service assembly name
     * @throws Exception
     *             if SU identifier is null, empty and non unique within the sa
     */
    protected void checkSUIdentifier(List<String> suIds, String suIdentifier, String saName)
            throws PetalsException {
        if (StringHelper.isNullOrEmpty(suIdentifier)) {
            throw new PetalsException("Name of service unit must be filled in the SA '" + saName
                    + "'");
        }
        if (suIds.contains(suIdentifier)) {
            throw new PetalsException("Name of service unit must be unique within the SA '"
                    + saName + "'");
        }
    }

    /**
     * Check target artifact zip and target component entries : must be non null
     * and non empty
     * 
     * @param artifactZip
     *            the artefact zip path into the sa archive
     * @param suIdentifier
     *            the service unit identifier
     * @param targetComponent
     *            the targeted component name
     * @throws Exception
     *             if target artifact zip or target component entries are null
     *             or empty
     */
    protected void checkTargetComponentAndZip(String artifactZip, String suIdentifier,
            String targetComponent) throws PetalsException {
        String msg = "";
        if (StringHelper.isNullOrEmpty(artifactZip)) {
            msg = "Target artifact zip of a service unit must be non null "
                    + "and non empty (SU name : " + suIdentifier + ").";
            log.error(msg);
            throw new PetalsException(msg);
        }
        if (StringHelper.isNullOrEmpty(targetComponent)) {
            msg = "Target component name of a service unit must be non null "
                    + "and non empty (SU name : " + suIdentifier + ").";
            log.error(msg);
            throw new PetalsException(msg);
        }
    }

    /**
     * Check that all declared su zip are presents in the sa package
     * 
     * @param artifactPath
     *            the artefact path into the sa archive
     * @param filePackage
     *            the service assembly package root
     * @throws Exception
     *             if at least one su is missing into the sa archive
     */
    protected void checkSUPackageInSAPackage(String artifactPath, File filePackage)
            throws PetalsException {
        File artifactFile = new File(filePackage, artifactPath);
        if (!artifactFile.exists()) {
            throw new PetalsException("Can't find artifact '" + artifactPath + "' in SA package  '"
                    + filePackage.getAbsolutePath() + "'");
        }
    }

    /**
     * Check if the SU is already deployed on the target component.
     * 
     * @param su
     *            the service unit production element
     * 
     * @return {@code true} if the SI is already deployed on the target
     *         component, {@code false} otherwise
     * @throws Exception
     *             if deploymentService.isDeployedServiceUnit fails
     */
    protected boolean isSUAlreadyDeployedOnTargetedComponent(ServiceUnit su) throws PetalsException {
        return isDeployedServiceUnit(su.getTarget().getComponentName(), su.getIdentification()
                .getName());
    }

    @Override
    protected String extractEntityName(Jbi descriptor) {
        return descriptor.getServiceAssembly().getIdentification().getName();
    }

    private boolean isDeployedServiceUnit(String componentName, String serviceUnitName) {

        boolean result = false;
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = adminService
                .getServiceAssemblyLifeCycles();
        for (Iterator<ServiceAssemblyLifeCycle> iter = saLifeCycles.values().iterator(); iter
                .hasNext()
                && !result;) {
            ServiceAssemblyLifeCycle salc = (ServiceAssemblyLifeCycle) iter.next();
            List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
            for (ServiceUnit unit : sus) {
                if (componentName.equals(unit.getTarget().getComponentName())
                        && serviceUnitName.equals(unit.getIdentification().getName())) {
                    result = true;
                }
            }
        }
        return result;
    }

}
