/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 INRIA, http://www.inria.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.transaction;

import java.rmi.RemoteException;

import javax.jbi.messaging.MessageExchange;
import javax.rmi.PortableRemoteObject;
import javax.transaction.HeuristicMixedException;
import javax.transaction.HeuristicRollbackException;
import javax.transaction.InvalidTransactionException;
import javax.transaction.NotSupportedException;
import javax.transaction.RollbackException;
import javax.transaction.Status;
import javax.transaction.SystemException;
import javax.transaction.Transaction;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.jotm.Coordinator;
import org.objectweb.jotm.Current;
import org.objectweb.jotm.InternalTransactionContext;
import org.objectweb.jotm.Terminator;
import org.objectweb.jotm.TransactionContext;
import org.objectweb.jotm.TransactionFactoryImpl;
import org.objectweb.jotm.TransactionImpl;
import org.objectweb.jotm.Xid;
import org.objectweb.jotm.XidImpl;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.util.LoggingUtil;

/**
 * The Transaction Manager for the PEtALS server
 * 
 * @author Nicolas Dolet, Philippe Merle - INRIA, Roland NAUDIN EBMWebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = javax.transaction.TransactionManager.class))
public class TransactionManagerImpl implements javax.transaction.TransactionManager {

    /**
     * Logging utilities
     */
    protected LoggingUtil log;

    @Monolog(name = "logger")
    protected Logger logger;

    private static Current current;

    // ////////////////////////////////////////////
    // TransactionManager implementation
    // ////////////////////////////////////////////

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#begin()
     */
    public void begin() throws NotSupportedException, SystemException {

        current.begin();

        // Creation of the global transaction coordinator
        TransactionContext localTC = current.getPropagationContext(false);
        Coordinator c;
        if (localTC.getCoordinator() == null) {
            try {
                c = (Coordinator) PortableRemoteObject.narrow(PortableRemoteObject.toStub(Current
                        .getJTM().recreate(localTC)), Coordinator.class);
            } catch (RemoteException e) {
                throw new SystemException(e.getMessage());
            }
            localTC.setCoordinator(c);
            localTC.setTerminator((Terminator) c);
        }
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#commit()
     */
    public void commit() throws RollbackException, HeuristicMixedException,
            HeuristicRollbackException, SecurityException, IllegalStateException, SystemException {
        current.commit();
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#getStatus()
     */
    public int getStatus() throws SystemException {
        return current.getStatus();
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#getTransaction()
     */
    public Transaction getTransaction() throws SystemException {
        return current.getTransaction();
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#resume(javax.transaction.Transaction)
     */
    public void resume(Transaction t) throws InvalidTransactionException, IllegalStateException,
            SystemException {
        current.resume(t);
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#rollback()
     */
    public void rollback() throws IllegalStateException, SecurityException, SystemException {
        current.rollback();
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#setRollbackOnly()
     */
    public void setRollbackOnly() throws IllegalStateException, SystemException {
        current.setRollbackOnly();
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#setTransactionTimeout(int)
     */
    public void setTransactionTimeout(int timeout) throws SystemException {
        current.setTransactionTimeout(timeout);
    }

    /*
     * (non-Javadoc)
     * @see javax.transaction.TransactionManager#suspend()
     */
    public Transaction suspend() throws SystemException {
        return current.suspend();
    }

    /**
     * Prepare the transaction context and attach it to the message
     * 
     * @param messageExchange
     *            The transacted message exchange
     * @throws SystemException
     */
    public static void encodeTransactionContext(MessageExchangeImpl messageExchange)
            throws SystemException {
        // If there is an active transaction, build a PetalsTransactionContext
        // to send it with the message
        TransactionContext currentTransactionContext = null;

        // Transaction from PEtALS CDK component are stored in properties of
        // the Message Exchange whereas others component may use the Thread
        // Local storing
        if (messageExchange.getProperty(MessageExchange.JTA_TRANSACTION_PROPERTY_NAME) instanceof TransactionImpl) {
            TransactionImpl transaction = (TransactionImpl) messageExchange
                    .getProperty(MessageExchange.JTA_TRANSACTION_PROPERTY_NAME);
            if (transaction != null && transaction.getStatus() == Status.STATUS_ACTIVE) {
                currentTransactionContext = transaction.getPropagationContext(false);
            }
        } else if (current.getStatus() == Status.STATUS_ACTIVE) {
            currentTransactionContext = current.getPropagationContext(false);
        }

        if (currentTransactionContext != null) {
            // Build a new PetalsTransactionContext
            PetalsTransactionContext tc = new PetalsTransactionContext();
            Xid xid = currentTransactionContext.getXid();
            tc.formatID = xid.getFormatId();
            tc.globalTransactionID = xid.getGlobalTransactionId();
            tc.branchQualifier = xid.getBranchQualifier();
            tc.coordinator = currentTransactionContext.getCoordinator();
            tc.timeOut = currentTransactionContext.getTimeout();
            // Attach the transaction context to the message exchange
            messageExchange.setProperty(MessageExchange.JTA_TRANSACTION_PROPERTY_NAME, tc);

            // Set the 'transacted' MessageExchangeImpl property
            messageExchange.setTransacted(true);
        }
    }

    /**
     * Get the transaction context from the message ,update the petals
     * transaction manager and set the Transaction to the bound property
     * 
     * @param messageExchange
     *            the transacted message
     */
    public static void decodeTransactionContext(MessageExchangeImpl messageExchange) {
        // Get the TransactionContext attached to the message
        PetalsTransactionContext tctx = (PetalsTransactionContext) messageExchange
                .getProperty(MessageExchange.JTA_TRANSACTION_PROPERTY_NAME);

        if (tctx != null) {
            // Build a new Internal Transaction Context from the received
            // PetalsTransactionContext
            Xid xid = new XidImpl(tctx.formatID, tctx.globalTransactionID, tctx.branchQualifier);
            TransactionContext localTC = new InternalTransactionContext(tctx.timeOut,
                    tctx.coordinator, (javax.transaction.xa.Xid) xid);

            // update current with the received TransactionContext for non CDK
            // components
            current.setPropagationContext(localTC, false);

            // update Message Exchange Property for CDK components
            messageExchange.setProperty(MessageExchange.JTA_TRANSACTION_PROPERTY_NAME,
                    new TransactionImpl(localTC));
        }

    }

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(logger);
        this.log.start();

        try {
            current = new Current(new TransactionFactoryImpl());
        } catch (RemoteException e) {
            // never thrown by JOTM
            e.printStackTrace();
        }
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.call();
    }

}
