/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AddressResolver.java,v 1.1 2005/07/22 10:24:27 alouis Exp $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing;

import java.util.ArrayList;
import java.util.List;

import javax.jbi.component.Component;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.communication.topology.TopologyService;
import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.messaging.registry.EndpointRegistry;
import org.ow2.petals.jbi.messaging.endpoint.AbstractEndpoint;
import org.ow2.petals.jbi.messaging.endpoint.InternalEndpoint;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.jbi.messaging.routing.strategy.EndpointChooserStrategy;
import org.ow2.petals.kernel.configuration.ConfigurationService;

/**
 * The <code>AddressResolver</code> resolves implicit addresses such as
 * <code>interface</code> or <code>service</code> into an
 * <code>endpoint</code>. Rules are used to help the
 * <code>AddressResolver</code> in the choice of the <code>endpoint</code>,
 * if more than one implement the implicit address.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * 
 */
public class AddressResolver {

    /**
     * the different kind of research of a endpoint: by service or by interface.
     */
    private static enum kindSearch {
        /**
         * the research by service.
         */
        SERVICE_SEARCH,

        /**
         * the research by interface.
         */
        INTERFACE_SEARCH
    }

    /**
     * the Configuration service.
     */
    private ConfigurationService configurationService;

    /**
     * the Endpoint service.
     */
    private EndpointRegistry EndpointRegistry;

    /**
     * The EndpointChooserStrategy
     */
    private EndpointChooserStrategy endpointChooserStrategy;

    /**
     * Creates a new instance of {@link AddressResolver}.
     * 
     * @param topologyService
     *            the Topology service
     * @param EndpointRegistry
     *            the Endpoint service
     * @throws RoutingException
     *             impossible to instance the address resolver
     */
    public AddressResolver(final ConfigurationService configurationService,
            final TopologyService topologyService, final EndpointRegistry EndpointRegistry)
            throws RoutingException {
        this.configurationService = configurationService;
        this.EndpointRegistry = EndpointRegistry;
        this.endpointChooserStrategy = new EndpointChooserStrategy(configurationService,
                topologyService);
    }

    /**
     * Resolve the address if necessary. Resolution consist in finding an
     * endpoint satisfying <code>interface</code> or <code>service</code>.
     * the source of the exchange and the exchange help the
     * <code>AddressResolver</code> to find the endpoint. The
     * <code>containerName</code> and <code>ComponentName</code> properties
     * are set. If <code>containerName</code> and <code>ComponentName</code>
     * already exist, do nothing. The check of acceptance from the two part is
     * done here (the Component method isExchangeWithXXXOkay()).
     * 
     * @param source
     *            the component context
     * @param exchange
     *            the exchange
     * @throws RoutingException
     *             no destination can be found, or problem accessing the
     *             registry
     */
    public void resolveAddress(final ComponentContextImpl source, final MessageExchangeImpl exchange)
            throws RoutingException {

        // get message exchange values
        ServiceEndpoint givenEndpoint = exchange.getEndpoint();
        QName givenServiceName = exchange.getService();
        QName givenInterfaceName = exchange.getInterfaceName();

        // Construct an array of potential matching endpoints.
        // If the endpoint has been set in the messageExchange,
        // this array contains only one element, which is this endpoint.
        ServiceEndpoint targetedEndpoint = null;

        if (givenEndpoint != null) {
            /*
             * Case 1 : The endpoint has been specified in the message exchange,
             * use it
             */
            targetedEndpoint = this.getTargetedEndpointFromGivenEndpoint(givenEndpoint);

        } else if (givenServiceName != null) {
            /*
             * Case 2 : The endpoint is not specified but the service name is.
             * Get all the endpoints which are satisfying the given service
             */
            targetedEndpoint = this.getTargetedEndpointFromGivenServiceName(givenServiceName,
                    exchange, source.getComponent());

        } else if (givenInterfaceName != null) {
            /*
             * Case 3 : Nor the endpoint nor the service have been specified in
             * the message exchange. Get all the endpoints which resolves the
             * given interface;
             */
            targetedEndpoint = this.getTargetedEndpointFromGivenInterfaceName(givenInterfaceName,
                    exchange, source.getComponent());

        }

        if (targetedEndpoint == null) {
            throw new RoutingException(
                    "Failed to find a destination for the MessageExchange with id: "
                            + exchange.getExchangeId());
        }

        // Now, we can set the real endpoint that will
        // serves the MessageExchange. Target endpoint can be null if it has not
        // been found...
        exchange.setEndpoint(targetedEndpoint);
    }

    /**
     * Get the targeted endpoint from a given endpoint.
     * 
     * @param givenEndpoint
     *            the endpoint in the message exchange. May be null
     * @return the targeted endpoint. May be null
     * @throws RoutingException
     *             impossible to get the endpoint
     */
    private ServiceEndpoint getTargetedEndpointFromGivenEndpoint(final ServiceEndpoint givenEndpoint)
            throws RoutingException {
        ServiceEndpoint targetedEndpoint = null;
        if (givenEndpoint != null) {
            // try to resolve address with a connection
            // TODO minimize registry access, refactor
            if (givenEndpoint instanceof AbstractEndpoint
                    && ((AbstractEndpoint) givenEndpoint).getType() == AbstractEndpoint.EndpointType.INTERNAL) {
                // Assume it is a real endpoint
                // FIXME don't check if it is still registered
                targetedEndpoint = givenEndpoint;
            } else {
                targetedEndpoint = this.resolveLinkedAddressForEndpoint(givenEndpoint
                        .getServiceName(), givenEndpoint.getEndpointName());

                // if not a connection, try to retrieve the real endpoint object
                // from the registry
                if (targetedEndpoint == null) {
                    targetedEndpoint = this.findEndpointInRegistry(givenEndpoint);
                }
            }
        }
        return targetedEndpoint;
    }

    /**
     * Find the endpoint in registry.
     * 
     * @param givenEndpoint
     *            the given endpoint. May be null.
     * @return the targeted endpoint. May be null.
     * @throws RoutingException
     *             Impossible to find the endpoint
     */
    private ServiceEndpoint findEndpointInRegistry(final ServiceEndpoint givenEndpoint)
            throws RoutingException {
        ServiceEndpoint targetedEndpoint = null;
        if (givenEndpoint != null) {
            targetedEndpoint = this.EndpointRegistry.getEndpoint(givenEndpoint.getServiceName(),
                    givenEndpoint.getEndpointName());
            // no endpoint found, we can not find a destination
            if (targetedEndpoint == null) {
                throw new RoutingException("The specified endpoint (" + givenEndpoint
                        + ") does not match a registered endpoint.");
            }
        }
        return targetedEndpoint;
    }

    /**
     * Get the targeted endpoint from a given service name.
     * 
     * @param givenServiceName
     *            the given Service Name in the message exchange. May be null
     * @param exchange
     *            the exchange
     * @param component
     *            the component
     * @return the targeted endpoint following the specific strategy used. May
     *         be null
     * @throws RoutingException
     *             impossible to get the endpoint
     */
    private ServiceEndpoint getTargetedEndpointFromGivenServiceName(final QName givenServiceName,
            final MessageExchange exchange, final Component component) throws RoutingException {
        ServiceEndpoint targetedEndpoint = null;
        if (givenServiceName != null) {
            List<ServiceEndpoint> endpoints = null;
            endpoints = this.getEnabledEndpoints(AddressResolver.kindSearch.SERVICE_SEARCH,
                    component, givenServiceName, exchange);

            // if there is several endpoints that matching
            // the given information, we have to choose one.
            targetedEndpoint = endpointChooserStrategy.choose(endpoints, exchange);

            if (targetedEndpoint == null) {
                throw new RoutingException("No endpoint found for the specified service : "
                        + givenServiceName);
            } else {
                //resolve the endpoint if it is a connection
                targetedEndpoint = getTargetedEndpointFromGivenEndpoint(targetedEndpoint);
            }
        }
        return targetedEndpoint;
    }

    /**
     * Get the targeted endpoint from a given interface name.
     * 
     * @param givenInterfaceName
     *            the given interface Name in the message exchange. May be null
     * @param exchange
     *            the exchange.
     * @param component
     *            the component.
     * @return the targeted endpoint following the specific strategy used. May
     *         be null
     * @throws RoutingException
     *             impossible to get the endpoint
     */
    private ServiceEndpoint getTargetedEndpointFromGivenInterfaceName(
            final QName givenInterfaceName, final MessageExchange exchange,
            final Component component) throws RoutingException {
        ServiceEndpoint targetedEndpoint = null;
        List<ServiceEndpoint> endpoints = null;
        if (givenInterfaceName != null) {
            // find an Endpoint with the specified InterfaceName
            endpoints = this.getEnabledEndpoints(AddressResolver.kindSearch.INTERFACE_SEARCH,
                    component, givenInterfaceName, exchange);

            // if there is several endpoints matching
            // the given information, we have to choose one.
            targetedEndpoint = endpointChooserStrategy.choose(endpoints, exchange);

            if (targetedEndpoint == null) {
                throw new RoutingException("No endpoint found for the specified interface : "
                        + givenInterfaceName + ".");
            }
        }
        return targetedEndpoint;
    }

    /**
     * Retrieve a potential mapping between a fully qualified endpoint name
     * (service name + endpoint name) and a real endpoint.
     * 
     * @param serviceName
     *            the service name
     * @param endpointName
     *            the endpoint name
     * @throws RoutingException
     *             impossible to resolve the address
     * @return the linked endpoint, null if no connection retrieved
     */
    protected ServiceEndpoint resolveLinkedAddressForEndpoint(final QName serviceName,
            final String endpointName) throws RoutingException {
        return this.EndpointRegistry.getEndpoint(serviceName, endpointName);
    }

    /**
     * Return only the enabled endPoints among the potentials endpoints.
     * 
     * @param type
     *            the type of research (by service or interface)
     * @param consumer
     *            the consumer
     * @param givenName
     *            the given service or interface name
     * @param exchange
     *            the exchange
     * @return the list of valide endpoints
     */
    public List<ServiceEndpoint> getEnabledEndpoints(final kindSearch type,
            final Component consumer, final QName givenName, final MessageExchange exchange) {
        ServiceEndpoint[] potentielEndpoints = null;
        if (type == AddressResolver.kindSearch.SERVICE_SEARCH) {
            potentielEndpoints = this.EndpointRegistry.getInternalEndpointsForService(givenName);
        } else {
            potentielEndpoints = this.EndpointRegistry.getInternalEndpointsForInterface(givenName);
        }
        List<ServiceEndpoint> validEndpoints = new ArrayList<ServiceEndpoint>();
        for (ServiceEndpoint providerEP : potentielEndpoints) {
            if (this.controlAcceptationExchange(consumer, (InternalEndpoint) providerEP, exchange)) {
                validEndpoints.add(providerEP);
            }
        }
        return validEndpoints;
    }

    /**
     * Control that the exchange is possible. Verify if the consumer and
     * provider are ok to realize the exchange.
     * 
     * @param consumer
     *            the consumer
     * @param providerEP
     *            the provider
     * @param exchange
     *            the exchange
     * @return true if ok
     */
    private boolean controlAcceptationExchange(final Component consumer,
            final InternalEndpoint providerEP, final MessageExchange exchange) {
        boolean result = false;
        // test if isExchangeWithXXXOkay has to be used.
        if (configurationService.getContainerConfiguration().isExchangeValidation()) {
            // apply isExchangeWithXXXOkay
            if (consumer.isExchangeWithProviderOkay(providerEP, exchange)
                    && providerEP.getEndpointRegistry().isExchangeWithConsumerOkayForComponent(
                            providerEP, exchange)) {
                result = true;
            }
        } else {
            result = true;
        }
        return result;
    }
}
