/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing.strategy;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.ow2.petals.jbi.messaging.routing.RoutingException;

/**
 * @author Roland NAUDIN - EBM WebSourcing
 * 
 */
public class RandomStrategy implements Strategy {

    private static final int NUMBER_STRATEGY_PARAMETERS = 3;

    /**
     * The endpoint chooser strategy instance
     */
    public EndpointChooserStrategy endpointChooserStrategy;

    private float localWeighting;

    private float remoteActiveWeighting;

    private float remoteInactiveWeighting;

    /**
     * Random used to elect the endpoint
     */
    private Random random = null;

    public RandomStrategy(EndpointChooserStrategy endpointChooserStrategy) {
        this.random = new Random();
        this.endpointChooserStrategy = endpointChooserStrategy;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.jbi.messaging.routing.strategy.Strategy#validateStrategyParameters(java.util.List)
     */
    public void validateStrategyParameters(List<String> strategyParameters) throws RoutingException {

        try {
            if (strategyParameters.size() == NUMBER_STRATEGY_PARAMETERS) {
                Iterator<String> it = strategyParameters.iterator();
                localWeighting = Float.valueOf(it.next());
                remoteActiveWeighting = Float.valueOf(it.next());
                remoteInactiveWeighting = Float.valueOf(it.next());
            } else {
                throw new RoutingException(
                        "Invalid parameters: 3 parameters are required by the random routing strategy");
            }
        } catch (ClassCastException e) {
            throw new RoutingException(
                    "Impossible to convert values of parameters for the random routing strategy");
        }

        if (localWeighting < 0 || remoteActiveWeighting < 0 || remoteInactiveWeighting < 0) {
            throw new RoutingException("Negative parameter not allowed by the random strategy");
        }

        if (localWeighting == 0 && remoteActiveWeighting == 0 && remoteInactiveWeighting == 0) {
            throw new RoutingException("Weighting 0,0,0 is not allowed by the random strategy");
        }

    }

    /**
     * The random strategy. This strategy chooses an endpoint according to
     * defined weightings. The endpoints that have the strongest weighting have
     * more chances to be elected than others.
     * 
     * @see org.ow2.petals.jbi.messaging.routing.strategy.Strategy#electEndpoint(java.util.List,
     *      javax.jbi.messaging.MessageExchange)
     */
    public ServiceEndpoint electEndpoint(List<ServiceEndpoint> endpoints, MessageExchange exchange)
            throws RoutingException {

        Map<ServiceEndpoint, Float> weightingEndpoints = this.endpointChooserStrategy
                .getWeightingEndpoints(endpoints, localWeighting, remoteActiveWeighting,
                        remoteInactiveWeighting);

        float factor = sumWeightings(weightingEndpoints) * random.nextFloat();

        ServiceEndpoint provider = null;
        float idMin = 0;
        float idMax = 0;
        ServiceEndpoint result = null;

        Iterator<ServiceEndpoint> it = endpoints.iterator();
        if (factor == 0) {
            result = it.next();
        }

        while ((result == null) && (it.hasNext())) {
            provider = it.next();
            idMax = idMin + weightingEndpoints.get(provider).floatValue();
            if (idMin <= factor && factor < idMax) {
                result = provider;
            }
            idMin = idMax;
        }
        
        return result;
    }

    /**
     * Compute the sum of the weighting of all endpoints.
     * 
     * @param weightings
     *            the weightings
     * @return the sum of the weightings
     */
    private static float sumWeightings(final Map<ServiceEndpoint, Float> weightings) {
        float sum = 0;
        for (Float weighting : weightings.values()) {
            sum = sum + weighting;
        }

        return sum;
    }

}
