/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.configuration;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map;
import java.util.Properties;

import javax.xml.bind.JAXBException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.kernel.configuration.ContainerConfiguration.ContainerState;
import org.ow2.petals.topology.TopologyBuilder;
import org.ow2.petals.topology.TopologyException;
import org.ow2.petals.topology.TopologyHelper;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.DomainMode;
import org.ow2.petals.topology.generated.JoramService;
import org.ow2.petals.topology.generated.Subdomain;
import org.ow2.petals.topology.generated.Topology;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.StringHelper;

/**
 * Class implementing the {@link ConfigurationService} interface.
 * 
 * @author Roland NAUDIN - EBMWebsourcing
 * @since Petals 2.0
 */

@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class))
public class ConfigurationServiceImpl implements ConfigurationService {

    private class SubDomainAndContainerNames {

        protected String subDomainName;

        protected String containerName;
    }

    /**
     * Logging utilities
     */
    protected LoggingUtil log;

    @Monolog(name = "logger")
    protected Logger logger;

    private ContainerConfiguration containerConfiguration;

    private SubDomainConfiguration subDomainConfiguration;

    private DomainConfiguration domainConfiguration;

    private Properties serverProperties;

    private Topology topology;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#getContainerConfiguration()
     */
    public ContainerConfiguration getContainerConfiguration() {
        this.log.call();
        return this.containerConfiguration;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#getSubDomainConfiguration()
     */
    public SubDomainConfiguration getSubDomainConfiguration() {
        this.log.call();
        return this.subDomainConfiguration;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#getDomainConfiguration()
     */
    public DomainConfiguration getDomainConfiguration() {
        this.log.call();
        return this.domainConfiguration;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#getStaticTopology()
     */
    public Topology getStaticTopology() throws ConfigurationException {
        this.log.call();
        if (this.domainConfiguration.getMode().equals(DomainConfiguration.DomainMode.DYNAMIC)) {
            throw new ConfigurationException("The domain is not in static mode");
        }
        return this.topology;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#setContainerState(org.ow2.petals.kernel.configuration.ContainerConfiguration.ContainerState)
     */
    public void setContainerState(ContainerState state) {
        this.containerConfiguration.setState(state);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.kernel.configuration.ConfigurationService#verifyContainerTopology(org.ow2.petals.kernel.configuration.ContainerConfiguration,
     *      org.ow2.petals.kernel.configuration.SubDomainConfiguration,
     *      org.ow2.petals.kernel.configuration.DomainConfiguration)
     */
    public void verifyContainerTopology(final ContainerConfiguration newContainerConfiguration,
            final SubDomainConfiguration newSubDomainConfiguration,
            final DomainConfiguration newDomainConfiguration) throws TopologyException {
        this.log.start();

        this.verifyDomain(newDomainConfiguration);

        this.verifySubDomain(newSubDomainConfiguration);

        this.verifyContainer(newContainerConfiguration);

        this.log.end();
    }

    /**
     * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws Exception {
        this.log = new LoggingUtil(this.logger);
        this.log.start();

        try {
            // retrieve resources
            URL serverPropsURL = this.getClass().getResource(SERVER_PROPS_FILE);
            if (serverPropsURL == null) {
                throw new IOException("Failed to reach the resource '" + SERVER_PROPS_FILE + "'");
            }
            
            File serverPropsFile = new File(serverPropsURL.getPath());
            URL topologyURL = this.getClass().getResource(TOPOLOGY_FILE);
            if (topologyURL == null) {
                throw new IOException("Failed to reach the resource '" + TOPOLOGY_FILE + "'");
            }
            
            File topologyFile = new File(topologyURL.getPath());
            this.containerConfiguration = new ContainerConfiguration();
            this.containerConfiguration.init();
            this.subDomainConfiguration = new SubDomainConfiguration();
            this.domainConfiguration = new DomainConfiguration();

            this.createConfiguration(topologyFile, serverPropsFile);

        } catch (IOException e) {
            this.log.error("Failed to start Configuration service - " + e.getClass().getSimpleName()
                    + " - " + e.getMessage());
            throw e;
        } catch (ConfigurationException e) {
            this.log.error("Failed to start Configuration service - " + e.getClass().getSimpleName()
                    + " - " + e.getMessage());
            throw e;
        } catch (JAXBException e) {
            this.log.error("Failed to start Configuration service - " + e.getClass().getSimpleName()
                    + " - " + e.getMessage());
            throw e;
        } catch (TopologyException e) {
            this.log.error("Failed to start Configuration service - " + e.getClass().getSimpleName()
                    + " - " + e.getMessage());
            throw e;
        }

        this.log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        this.log.call();
    }

    /**
     * Create the container configuration based on the topology and server
     * files. Basic consistency is controlled between the 2 configuration files.
     * The {@code serverPropsFile} and <{@code topologyFile} fields must be non
     * null.
     * 
     * @param topologyFile
     *            The topology.xml file
     * @param serverPropsFile
     *            The server.properties file
     * 
     * @throws JAXBException
     * @throws TopologyException
     * @throws IOException
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    private void createConfiguration(File topologyFile, File serverPropsFile) throws JAXBException,
            TopologyException, IOException, ConfigurationException {
        this.log.start();

        // build the topology object
        this.topology = TopologyBuilder.createTopology(topologyFile);

        // retrieve the server properties
        this.serverProperties = new Properties();
        this.serverProperties.load(new FileInputStream(serverPropsFile));

        String containerName = this.serverProperties.getProperty(PROPERTY_CONTAINER_NAME);
        if (containerName == null) {
            containerName = System.getProperty(PROPERTY_CONTAINER_NAME);
        }

        final SubDomainAndContainerNames values = this.validateTopology(containerName);

        this.fillGlobalTopology();

        this.fillSubDomainTopology(values.subDomainName);

        this.fillLocalTopology(values.containerName);

        this.fillProperties();

        // in standalone mode the container is always started
        if (org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode.STANDALONE
                .equals(this.domainConfiguration.getMode())) {
            this.containerConfiguration.setState(ContainerState.STARTED);
        }

        this.log.debug("Configuration used :");
        this.log.debug(this.containerConfiguration);

        this.log.end();
    }

    /**
     * Validate the consistency of the topology. The {@code topology} and the
     * {@code containerConfiguration} fields must be non {@code null}.
     * Moreover, if the domain is a standalone domain,
     * <code>containerName</code> and <subDomainName> are updated according to
     * the topology.
     * 
     * @param containerName
     *            The name of the local container retrieved from the
     *            environment, can be {@ null}
     * @return The container name and the sub-domain name of the provided
     *         container.
     * @throws ConfigurationException
     */
    private SubDomainAndContainerNames validateTopology(final String containerName)
            throws ConfigurationException {
        this.log.start();

        final SubDomainAndContainerNames resultValues = new SubDomainAndContainerNames();

        if (DomainMode.STANDALONE.equals(this.topology.getDomain().getMode())) {
            if (this.topology.getDomain().getSubDomain().size() > 1
                    || this.topology.getDomain().getSubDomain().get(0).getContainer().size() > 1)
                throw new ConfigurationException(
                        "Standalone mode accepts only one sub-domain and one container definition.");
        }

        // validate domain consistency
        if (!DomainMode.STANDALONE.equals(this.topology.getDomain().getMode())) {
            if (containerName == null) {
                throw new ConfigurationException("The property '" + PROPERTY_CONTAINER_NAME
                        + "' must be set in distributed mode");
            }
            if (TopologyHelper.findContainer(containerName, this.topology) == null) {
                throw new ConfigurationException("Cannot find the definition for the container "
                        + containerName + " in the topology.");
            }
            if (DomainMode.STATIC.equals(this.topology.getDomain().getMode())) {
                this.validateStaticTopology();
            }
            resultValues.containerName = containerName;
        } else {
            resultValues.containerName = this.topology.getDomain().getSubDomain().get(0)
                    .getContainer().get(0).getName();
        }
        resultValues.subDomainName = TopologyHelper.findSubdomain(
                TopologyHelper.findContainer(resultValues.containerName, this.topology),
                this.topology).getName();

        // validate JNDI consistency
        if (this.topology.getDomain().getJndi() != null) {
            if ("".equals(this.topology.getDomain().getJndi().getFactory().trim())) {
                throw new ConfigurationException(
                        "JNDI factory classname must be set to reference an external JNDI server");
            }
            if (this.topology.getDomain().getJndi().getHost() == null
                    || this.topology.getDomain().getJndi().getPort() <= 0) {
                throw new ConfigurationException(
                        "JNDI host and port must be set to reference an external JNDI server with factory : '"
                                + this.topology.getDomain().getJndi().getFactory() + "'");
            }
        }

        // validate containers configuration against the domain mode
        if (!DomainMode.STANDALONE.equals(this.topology.getDomain().getMode())) {
            Collection<Container> containers = TopologyHelper.findAllContainers(this.topology);
            for (Container container : containers) {
                if (container.getNetworkservice() == null) {
                    throw new ConfigurationException(
                            "Network service must be defined in the container '"
                                    + container.getName() + "' configuration");
                }
                if (container.getJoramservice() == null) {
                    throw new ConfigurationException(
                            "Joram service must be defined in the container '"
                                    + container.getName() + "' configuration");
                }
                if (container.getDreamservice() == null) {
                    throw new ConfigurationException(
                            "Dream service must be defined in the container '"
                                    + container.getName() + "' configuration");
                }
            }
        }

        this.log.end();
        return resultValues;
    }

    /**
     * Validate the consistency of the static topology
     * 
     * @throws ConfigurationException
     */
    private void validateStaticTopology() throws ConfigurationException {
        this.log.start();

        // Check all the sub domains names
        Collection<String> subdomainNameList = new HashSet<String>();
        for (Subdomain subdomain : this.topology.getDomain().getSubDomain()) {
            if (!subdomainNameList.add(subdomain.getName())) {
                throw new ConfigurationException("Duplicate sub-domain name '"
                        + subdomain.getName() + "'");
            }
        }

        // Check all the containers
        Collection<String> nameList = new HashSet<String>();
        Map<String, HashSet<Integer>> portMap = new Hashtable<String, HashSet<Integer>>();
        for (Container container : TopologyHelper.findAllContainers(this.topology)) {
            if (container.getHost() == null) {
                throw new ConfigurationException("The container named '" + container.getName()
                        + "' must declare its host element in static distributed mode");
            }

            HashSet<Integer> portList;
            if (!portMap.containsKey(container.getHost())) {
                portList = new HashSet<Integer>();
                portMap.put(container.getHost(), portList);
            } else {
                portList = portMap.get(container.getHost());
            }
            if (!portList.add(container.getNetworkservice().getPort())) {
                throw new ConfigurationException("Duplicate port "
                        + container.getNetworkservice().getPort() + " on host '"
                        + container.getHost() + "'");
            }
            if (container.getJmxservice().getRmiport() != 0) {
                if (!portList.add(container.getJmxservice().getRmiport())) {
                    throw new ConfigurationException("Duplicate port "
                            + container.getJmxservice().getRmiport() + " on host '"
                            + container.getHost() + "'");
                }
            }
            if (!portList.add(container.getJoramservice().getDomainPort())) {
                throw new ConfigurationException("Duplicate port "
                        + container.getJoramservice().getDomainPort() + " on host '"
                        + container.getHost() + "'");
            }
            if (container.getJoramservice().getTcpPort() != null) {
                if (!portList.add(container.getJoramservice().getTcpPort())) {
                    throw new ConfigurationException("Duplicate port "
                            + container.getJoramservice().getTcpPort() + " on host '"
                            + container.getHost() + "'");
                }
            }
            if (!portList.add(container.getDreamservice().getPort())) {
                throw new ConfigurationException("Duplicate port "
                        + container.getDreamservice().getPort() + " on host '"
                        + container.getHost() + "'");
            }
            if (!nameList.add(container.getName())) {
                throw new ConfigurationException("Duplicate container name '" + container.getName()
                        + "'");
            }
        }

        this.log.end();
    }

    /**
     * Fill the global topology.
     */
    private void fillGlobalTopology() {
        this.log.start();

        // domain mode
        this.domainConfiguration.setName(this.topology.getDomain().getName());
        this.domainConfiguration.setDescription(this.topology.getDomain().getDescription());
        if (DomainMode.STANDALONE.equals(this.topology.getDomain().getMode())) {
            this.domainConfiguration
                    .setMode(org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode.STANDALONE);
            // set the default value of the routing strategy of standalone mode
            this.containerConfiguration.setRoutingStrategy(Router.DEFAULT_STANDALONE_STRATEGY);
        } else if (DomainMode.STATIC.equals(this.topology.getDomain().getMode())) {
            this.domainConfiguration
                    .setMode(org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode.STATIC);
        } else {
            this.domainConfiguration
                    .setMode(org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode.DYNAMIC);
        }

        // domain network
        if (!DomainMode.STANDALONE.equals(this.topology.getDomain().getMode())) {
            if (this.topology.getDomain().getMulticast() != null) {
                this.domainConfiguration.setNetworkMuticastIp(this.topology.getDomain()
                        .getMulticast().getIp());
                this.domainConfiguration.setNetworkMuticastPort(this.topology.getDomain()
                        .getMulticast().getPort());
            } else {
                // fill the map of unicast network addresses
                Collection<Container> topologyCollection = TopologyHelper
                        .findAllContainers(this.topology);
                Map<String, String> networkUnicastAddressMap = new Hashtable<String, String>();
                for (Container container : topologyCollection) {
                    networkUnicastAddressMap.put(container.getName(), container.getHost() + "["
                            + container.getNetworkservice().getPort() + "]");
                }
                this.domainConfiguration.setNetworkUnicastAddressMap(networkUnicastAddressMap);
            }
        }

        // domain JNDI
        if (this.topology.getDomain().getJndi() != null) {
            this.domainConfiguration.setJndiFactory(this.topology.getDomain().getJndi()
                    .getFactory());
            this.domainConfiguration.setJndiHost(this.topology.getDomain().getJndi().getHost());
            this.domainConfiguration.setJndiPort(this.topology.getDomain().getJndi().getPort());
        }

        this.log.end();
    }

    /**
     * Fill the sub-domain configuration with its topology part. The
     * {@code subDomainConfiguration} and {@code topology} fields must be non
     * {@code null}.
     * 
     * @param subDomainName
     *            The sub-domain name. Must be non null.
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    private void fillSubDomainTopology(final String subDomainName) throws ConfigurationException {
        this.log.start();

        Subdomain subDomain = TopologyHelper.findSubdomain(subDomainName, this.topology);

        if (subDomain == null) {
            throw new ConfigurationException(
                    "Failed to retrieve the sub-domain configuration for the sub-domain named '"
                            + subDomainName + "'");
        }

        this.subDomainConfiguration.setName(subDomainName);
        this.subDomainConfiguration.setDescription(subDomain.getDescription());
        if (subDomain.isNetworkTimeSynchronized() != null) {
            this.subDomainConfiguration.setNetworkTimeSynchronized(subDomain
                    .isNetworkTimeSynchronized().booleanValue());
        }

        this.log.end();
    }

    /**
     * Fill the container configuration with its topology part. The
     * {@code containerConfiguration} and {@code topology} fields must be non
     * {@code null}.
     * 
     * @param container
     *            name The local container name
     * @throws ConfigurationException
     *             Configuration consistency error
     */
    private void fillLocalTopology(String containerName) throws ConfigurationException {
        this.log.start();

        Container localContainer = TopologyHelper.findContainer(containerName, this.topology);

        if (localContainer == null) {
            throw new ConfigurationException(
                    "Failed to retrieve the container configuration for the container named '"
                            + containerName + "'");
        }

        this.containerConfiguration.setName(containerName);
        this.containerConfiguration.setDescription(localContainer.getDescription());

        this.containerConfiguration.setSubdomainName(TopologyHelper.findSubdomain(localContainer,
                this.topology).getName());

        this.containerConfiguration.setUser(localContainer.getUser());
        this.containerConfiguration.setPassword(localContainer.getPassword());

        if (localContainer.getHost() != null) {
            this.containerConfiguration.setHost(localContainer.getHost());
        }

        // Network service, can be null
        if (localContainer.getNetworkservice() != null) {
            this.containerConfiguration.setNetworkBindingPort(localContainer.getNetworkservice()
                    .getPort());
        }

        // JMX Service
        this.containerConfiguration.setJmxRMIConnectorPort(localContainer.getJmxservice()
                .getRmiport());

        // JORAM Service
        final JoramService joramService = localContainer.getJoramservice();
        if (joramService != null) {
            this.containerConfiguration.setJoramDomainPort(joramService.getDomainPort());
            if (joramService.getTcpPort() != null) {
                this.containerConfiguration.setJoramTCPPort(joramService.getTcpPort());
            }
        }

        // DREAM Service
        if (localContainer.getDreamservice() != null) {
            this.containerConfiguration.setDreamTCPPort(localContainer.getDreamservice().getPort());
        }

        this.log.end();
    }

    /**
     * Fill the container configuration properties with its global properties part. The
     * {@code containerConfiguration} and {@code serverProperties} fields must
     * be non {@code null}.
     */
    private void fillProperties() {
        this.log.start();
        Map<String, String> userProperties = new HashMap<String, String>();
        for (Object key : this.serverProperties.keySet()) {
            if (key instanceof String) {
                if (((String) key).equals(PROPERTY_REPOSITORY_PATH)) {
                    this.containerConfiguration.setRepositoryDirectoryPath(this.serverProperties
                            .getProperty((String) key));
                } else if (((String) key).equals(PROPERTY_EXCHANGE_VALIDATION)) {
                    this.containerConfiguration.setExchangeValidation(new Boolean(
                            this.serverProperties.getProperty((String) key)).booleanValue());
                } else if (((String) key).equals(PROPERTY_TRANSPORT_QOS)) {
                    this.containerConfiguration.setTransportQOS(this.serverProperties
                            .getProperty((String) key));
                } else if (((String) key).equals(PROPERTY_ROUTING_STRATEGY)) {
                    this.containerConfiguration.setRoutingStrategy(this.serverProperties
                            .getProperty((String) key));
                } else if (((String) key).equals(PROPERTY_TASK_TIMEOUT)) {
                    this.containerConfiguration.setTaskTimeout(new Long(this.serverProperties
                            .getProperty((String) key)).longValue());
                } else if (((String) key).equals(PROPERTY_CLASSLOADER_ISOLATED)) {
                    this.containerConfiguration.setIsolatedJBIClassLoaders(new Boolean(
                            this.serverProperties.getProperty((String) key)).booleanValue());
                } else if (((String) key).startsWith(PROPERTY_USER_CONFIG_PREFIX + ".")) {
                    userProperties.put((String)key, this.serverProperties.getProperty((String) key));
                }
            }
        }
        this.containerConfiguration.setUserConfiguration(userProperties);

        this.log.end();
    }

    /**
     * Verify the given new domain configuration against the topology.
     * 
     * @param newDomainConfiguration
     * @throws TopologyException
     */
    private void verifyDomain(DomainConfiguration newDomainConfiguration) throws TopologyException {
        this.log.debug("Check the domain configuration of the new Container...");

        if (!newDomainConfiguration.getMode().equals(this.domainConfiguration.getMode())) {
            throw new TopologyException("Domain mode mismatch");
        }

        // JNDI service
        if (!StringHelper.equal(newDomainConfiguration.getJndiFactory(), this.domainConfiguration
                .getJndiFactory())) {
            throw new TopologyException("JNDI factory mismatch");
        }
        if (!StringHelper.equal(newDomainConfiguration.getJndiHost(), this.domainConfiguration
                .getJndiHost())) {
            throw new TopologyException("JNDI Host mismatch");
        }
        if (newDomainConfiguration.getJndiPort() != this.domainConfiguration.getJndiPort()) {
            throw new TopologyException("JNDI Port mismatch");
        }
        this.log.debug("Domain configuration of the new Container is OK.");
    }

    /**
     * Verify the given new sub-domain configuration against the topology.
     * 
     * @param newDomainConfiguration
     * @throws TopologyException
     */
    private void verifySubDomain(final SubDomainConfiguration newSubDomainConfiguration)
            throws TopologyException {
        this.log.debug("Check the sub-domain configuration of the new Container...");

        if (DomainMode.STATIC.equals(this.topology.getDomain().getMode())) {
            Subdomain newSubdomain = TopologyHelper.findSubdomain(newSubDomainConfiguration
                    .getName(), this.topology);

            if (newSubdomain == null) {
                throw new TopologyException("Subdomain of the Container unknown");
            }
            if (newSubdomain.isNetworkTimeSynchronized() == null) {
                if (newSubDomainConfiguration.isNetworkTimeSynchronized() == true) {
                    throw new TopologyException("Network synchronization mismatch");
                }
            } else if (newSubdomain.isNetworkTimeSynchronized().booleanValue() != newSubDomainConfiguration
                    .isNetworkTimeSynchronized()) {
                throw new TopologyException("Network synchronization mismatch");
            }
        } else if (newSubDomainConfiguration.getName()
                .equals(this.subDomainConfiguration.getName())) {
            if (newSubDomainConfiguration.isNetworkTimeSynchronized() != this.subDomainConfiguration
                    .isNetworkTimeSynchronized()) {
                throw new TopologyException("Network synchronization mismatch");
            }
        }

        this.log.debug("Sub-domain configuration of the new Container is OK.");
    }

    /**
     * Verify the given new container configuration against the topology.
     * 
     * @param newContainerConfiguration
     * @throws TopologyException
     */
    private void verifyContainer(ContainerConfiguration newContainerConfiguration)
            throws TopologyException {

        this.log.debug("Check the container configuration of the new Container.");
        if (DomainMode.STATIC.equals(this.topology.getDomain().getMode())) {
            Container newContainer = TopologyHelper.findContainer(newContainerConfiguration
                    .getName(), this.topology);

            if (newContainer == null) {
                throw new TopologyException("Container name unknown");
            }
            this.log.debug("... check container " + newContainer.getName() + " ...");
            if (!StringHelper.equal(newContainer.getUser(), newContainerConfiguration.getUser())) {
                throw new TopologyException("User mismatch");
            }
            if (!StringHelper.equal(newContainer.getPassword(), newContainerConfiguration
                    .getPassword())) {
                throw new TopologyException("Password mismatch");
            }

            // network Service
            if (newContainer.getNetworkservice().getPort() != newContainerConfiguration
                    .getNetworkBindingPort()) {
                throw new TopologyException("Network binding port mismatch");
            }
            // JMX Service
            if (newContainer.getJmxservice().getRmiport() != newContainerConfiguration
                    .getJmxRMIConnectorPort()) {
                throw new TopologyException("JMX RMI connector port mismatch");
            }
            // JORAM service
            if (newContainer.getJoramservice().getDomainPort() != newContainerConfiguration
                    .getJoramDomainPort()) {
                throw new TopologyException("JORAM Domain port mismatch");
            }
            if (newContainer.getJoramservice().getTcpPort() != null
                    && newContainer.getJoramservice().getTcpPort().intValue() != newContainerConfiguration
                            .getJoramTCPPort()) {
                throw new TopologyException("JORAM TCP port mismatch");
            }

            // DREAM service
            if (newContainer.getDreamservice().getPort() != newContainerConfiguration
                    .getDreamTCPPort()) {
                throw new TopologyException("DREAM TCP port mismatch");
            }
        } else if (newContainerConfiguration.getHost()
                .equals(this.containerConfiguration.getHost())) {
            if (newContainerConfiguration.getName() == this.containerConfiguration.getName()) {
                throw new TopologyException("Duplicate container name '"
                        + this.containerConfiguration.getName() + "'");
            }

        }
        this.log.debug("Container configuration of the new Container is OK.");
    }
}
