/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.kernel.configuration;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.kernel.api.server.util.SystemUtil;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.transport.Transporter;

/**
 * This class holds the overall configuration of a PEtALS container. It stores
 * both the local topology information and the global server properties
 * information.<br>
 * The configuration service retrieves the configuration when it is started and
 * creates an instance of this class. The instance is exposed on the distributed
 * JNDI server.
 * 
 * @author alouis, Roland NAUDIN - eBMWebSourcing
 * @since Petals 2.0
 * 
 */
public final class ContainerConfiguration implements Serializable {

    /**
     * the "unique" ID for serialization
     */
    private static final long serialVersionUID = 6543213857L;

    /**
     * Container state enumeration.
     */
    public static enum ContainerState {
        /**
         * Started state
         */
        STARTED,
        /**
         * Stopped state
         */
        STOPPED,
        /**
         * unknown state
         */
        UNKNOWN
    };

    // ********** Global ************//

    /**
     * Subdomain name.
     */
    private String subdomainName;

    /**
     * Container name.
     */
    private String name;

    /**
     * Container description
     */
    private String description;

    /**
     * Host of the container.
     */
    private String host;

    /**
     * State of the container.
     */
    private ContainerState state;

    /**
     * Install directory.
     */
    private String installDirectoryPath;

    /**
     * Repository directory path.
     */
    private String repositoryDirectoryPath;

    /**
     * Exchange validation.<br>
     * If <code>true</code>, used by the Address resolver to know if a
     * component is agree to receive a exchange from a consumer.
     */
    private boolean exchangeValidation;

    /**
     * Transport Quality of Service. {@see Transporter}
     */
    private String transportQOS = Transporter.DEFAULT_POLICY;

    /**
     * Routing Strategy. {@see EndpointChooserStrategy}
     */
    private String routingStrategy = Router.DEFAULT_PLATFORM_STRATEGY;

    /**
     * The timeout for processing the installation and deployment tasks.
     */
    private long taskTimeout;

    /**
     * Flag indicating if the JBI classLoaders must be isolated from the
     * container ClassLoader.
     */
    private boolean isolatedJBIClassLoaders;

    /**
     * Login to connect to the node.
     */
    private String user;

    /**
     * Password to connect to the node.
     */
    private String password;

    // ********** Network ************//

    /**
     * Network port. Port used for PEtALS unicast messages.
     */
    private int networkBindingPort;

    // ************ JMX **************//

    /**
     * Port of the RMI adaptor for JMX.
     */
    private int jmxRMIConnectorPort;

    // ***** JOrAM Transporter *****//

    /**
     * Port of the Joram domain.
     */
    private int joramDomainPort;

    /**
     * Port of the Joram TCP communication.
     */
    private int joramTCPPort;

    // ***** DReAM Transporter *****//

    /**
     * Port of the Dream TCP communication.
     */
    private int dreamTCPPort;
    
    /**
     * The additional configuration in the server.properties defined by the
     * user. Get all the user.* key/value pairs.
     */
    private Map<String, String> userConfiguration;

    /**
     * 
     */
    public ContainerConfiguration() {
        super();
    }

    public void init() throws ConfigurationException {
        this.state = ContainerState.UNKNOWN;
        try {
            this.installDirectoryPath = SystemUtil.getPetalsInstallDirectory().getCanonicalPath();
        } catch (IOException e) {
            throw new ConfigurationException("The PEtALS install directory is not valid : "
                    + SystemUtil.getPetalsInstallDirectory().getAbsolutePath());
        }
        this.repositoryDirectoryPath = this.installDirectoryPath + File.separator
                + RepositoryService.DEFAULT_REPOSITORY_PATH;

    }
    
    public Map<String, String> getUserConfiguration() {
        return this.userConfiguration;
    }

    public String getSubdomainName() {
        return this.subdomainName;
    }

    public void setSubdomainName(final String subdomainName) {
        this.subdomainName = subdomainName;
    }

    public String getName() {
        return this.name;
    }

    public String getDescription() {
        return this.description;
    }

    public String getHost() {
        return this.host;
    }

    public ContainerState getState() {
        return this.state;
    }

    public String getInstallDirectoryPath() {
        return this.installDirectoryPath;
    }

    public String getRepositoryDirectoryPath() {
        return this.repositoryDirectoryPath;
    }

    public boolean isExchangeValidation() {
        return this.exchangeValidation;
    }

    public String getTransportQOS() {
        return this.transportQOS;
    }

    public String getRoutingStrategy() {
        return this.routingStrategy;
    }

    public long getTaskTimeout() {
        return this.taskTimeout;
    }

    public boolean isIsolatedJBIClassLoaders() {
        return this.isolatedJBIClassLoaders;
    }

    public int getNetworkBindingPort() {
        return this.networkBindingPort;
    }

    public String getUser() {
        return this.user;
    }

    public String getPassword() {
        return this.password;
    }

    public int getJmxRMIConnectorPort() {
        return this.jmxRMIConnectorPort;
    }

    public int getJoramDomainPort() {
        return this.joramDomainPort;
    }

    public int getJoramTCPPort() {
        return this.joramTCPPort;
    }

    public int getDreamTCPPort() {
        return this.dreamTCPPort;
    }

    public void setState(ContainerState state) {
        this.state = state;
    }

    @Override
    public String toString() {
        StringBuilder toStringBuilder = new StringBuilder();
        toStringBuilder.append("Container ").append(this.name).append(" on ").append(this.host)
                .append(" in sub-domain ").append(this.subdomainName);
        if (this.description != null) {
            toStringBuilder.append("\n\t\t\tDescription :\n\t\t\t  ").append(this.description);
        }
        toStringBuilder.append("\n\t\t\tConfiguration :\n\t\t\t  - InstallDirectory:").append(
                this.installDirectoryPath);
        toStringBuilder.append("\n\t\t\t  - RepositoryDirectory:").append(
                this.repositoryDirectoryPath);
        toStringBuilder.append("\n\t\t\t  - ExchangeValidation:").append(this.exchangeValidation);
        toStringBuilder.append("\n\t\t\t  - TransportQOS:").append(this.transportQOS);
        toStringBuilder.append("\n\t\t\t  - RoutingStrategy:").append(this.routingStrategy);
        toStringBuilder.append("\n\t\t\t  - TaskTimeout:").append(this.taskTimeout);
        toStringBuilder.append("\n\t\t\t  - User:")
                .append((this.user != null) ? this.user : "none");
        if (this.password != null) {
            StringBuffer hiddenPassword = new StringBuffer(this.password.length());
            while (hiddenPassword.length() < this.password.length()) {
                hiddenPassword.append("*");
            }
            toStringBuilder.append("\n\t\t\t  - Password:").append(hiddenPassword);
        } else {
            toStringBuilder.append("\n\t\t\t  - Password:none");
        }
        toStringBuilder.append("\n\t\t\t  - NetworkBindingPort:").append(
                (this.networkBindingPort != 0) ? this.networkBindingPort : "none");
        toStringBuilder.append("\n\t\t\t  - JMXRMIConnectionPort:")
                .append(this.jmxRMIConnectorPort);
        toStringBuilder.append("\n\t\t\t  - JoramDomainPort:").append(
                (this.joramDomainPort != 0) ? this.joramDomainPort : "none");
        toStringBuilder.append("\n\t\t\t  - JoramTCPPort:").append(
                (this.joramTCPPort != 0) ? this.joramTCPPort : "none");
        toStringBuilder.append("\n\t\t\t  - DreamTCPPort:").append(
                (this.dreamTCPPort != 0) ? this.dreamTCPPort : "none");
        toStringBuilder.append("\n\t\t\t  - State:").append(this.state.name());
        
        StringBuffer userBuffer = new StringBuffer();
        Iterator<String> iter = this.userConfiguration.keySet().iterator();
        while (iter.hasNext()) {
            String key = iter.next();
            userBuffer.append("\n\t\t\t\t").append(key).append('=').append(this.userConfiguration.get(key));
        }
        toStringBuilder.append("\n\t\t\t  - User properties:").append(userBuffer.toString());

        return toStringBuilder.toString();
    }

    /**
     * Duplicate the given container configuration in an exported form.
     * 
     * @param containerConfiguration
     *            The original container configuration
     * @return the exported form
     */
    public static ContainerConfiguration getExportedForm(
            ContainerConfiguration containerConfiguration) {
        ContainerConfiguration exportedForm = new ContainerConfiguration();

        exportedForm.subdomainName = containerConfiguration.subdomainName;
        exportedForm.name = containerConfiguration.name;
        exportedForm.description = containerConfiguration.description;
        exportedForm.host = containerConfiguration.host;
        exportedForm.exchangeValidation = containerConfiguration.exchangeValidation;
        exportedForm.transportQOS = containerConfiguration.transportQOS;
        exportedForm.routingStrategy = containerConfiguration.routingStrategy;
        exportedForm.taskTimeout = containerConfiguration.taskTimeout;
        exportedForm.isolatedJBIClassLoaders = containerConfiguration.isolatedJBIClassLoaders;
        exportedForm.user = containerConfiguration.user;
        exportedForm.password = containerConfiguration.password;
        exportedForm.networkBindingPort = containerConfiguration.networkBindingPort;
        exportedForm.jmxRMIConnectorPort = containerConfiguration.jmxRMIConnectorPort;
        exportedForm.joramDomainPort = containerConfiguration.joramDomainPort;
        exportedForm.joramTCPPort = containerConfiguration.joramTCPPort;
        exportedForm.dreamTCPPort = containerConfiguration.dreamTCPPort;
        exportedForm.state = containerConfiguration.state;
        exportedForm.userConfiguration = containerConfiguration.userConfiguration;

        return exportedForm;
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + this.subdomainName.hashCode();
        result = prime * result + ((this.name == null) ? 0 : this.name.hashCode());
        result = prime * result + ((this.description == null) ? 0 : this.description.hashCode());
        result = prime * result + this.dreamTCPPort;
        result = prime * result + (this.exchangeValidation ? 1231 : 1237);
        result = prime * result + (this.isolatedJBIClassLoaders ? 1231 : 1237);
        result = prime * result + ((this.host == null) ? 0 : this.host.hashCode());
        result = prime * result + ((this.user == null) ? 0 : this.user.hashCode());
        result = prime * result + ((this.password == null) ? 0 : this.password.hashCode());
        result = prime * result + this.jmxRMIConnectorPort;
        result = prime * result + this.joramDomainPort;
        result = prime * result + this.joramTCPPort;
        result = prime * result + this.networkBindingPort;
        result = prime * result
                + ((this.routingStrategy == null) ? 0 : this.routingStrategy.hashCode());
        result = prime * result + (int) (this.taskTimeout ^ (this.taskTimeout >>> 32));
        result = prime * result + ((this.transportQOS == null) ? 0 : this.transportQOS.hashCode());
        result = prime * result + (this.userConfiguration.hashCode()); 
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        boolean result = false;
        if (obj instanceof ContainerConfiguration) {
            ContainerConfiguration otherContainerConfiguration = (ContainerConfiguration) obj;
            result = (otherContainerConfiguration.name.equals(this.name)
                    && otherContainerConfiguration.host.equals(this.host)
                    && otherContainerConfiguration.exchangeValidation == this.exchangeValidation
                    && otherContainerConfiguration.transportQOS.equals(this.transportQOS)
                    && otherContainerConfiguration.routingStrategy.equals(this.routingStrategy)
                    && otherContainerConfiguration.taskTimeout == this.taskTimeout
                    && otherContainerConfiguration.isolatedJBIClassLoaders == this.isolatedJBIClassLoaders
                    && otherContainerConfiguration.networkBindingPort == this.networkBindingPort
                    && otherContainerConfiguration.user.equals(this.user)
                    && otherContainerConfiguration.password.equals(this.password)
                    && otherContainerConfiguration.jmxRMIConnectorPort == this.jmxRMIConnectorPort
                    && otherContainerConfiguration.joramDomainPort == this.joramDomainPort
                    && otherContainerConfiguration.joramTCPPort == this.joramTCPPort && otherContainerConfiguration.dreamTCPPort == this.dreamTCPPort
                    && otherContainerConfiguration.userConfiguration.equals(this.userConfiguration));
        }
        return result;
    }
    
    protected void setUserConfiguration(Map<String, String> userConfiguration) {
        this.userConfiguration = userConfiguration;
    }

    protected void setName(String name) {
        this.name = name;
    }

    protected void setHost(String host) {
        this.host = host;
    }

    protected void setDescription(String description) {
        this.description = description;
    }

    protected void setInstallDirectoryPath(String installDirectoryPath) {
        this.installDirectoryPath = installDirectoryPath;
    }

    protected void setRepositoryDirectoryPath(String repositoryDirectoryPath) {
        this.repositoryDirectoryPath = repositoryDirectoryPath;
    }

    protected void setExchangeValidation(boolean exchangeValidation) {
        this.exchangeValidation = exchangeValidation;
    }

    protected void setTransportQOS(String transportQOS) {
        this.transportQOS = transportQOS;
    }

    protected void setRoutingStrategy(String routingStrategy) {
        this.routingStrategy = routingStrategy;
    }

    protected void setTaskTimeout(long taskTimeout) {
        this.taskTimeout = taskTimeout;
    }

    protected void setIsolatedJBIClassLoaders(boolean isolatedJBIClassLoaders) {
        this.isolatedJBIClassLoaders = isolatedJBIClassLoaders;
    }

    protected void setNetworkBindingPort(int networkBindingPort) {
        this.networkBindingPort = networkBindingPort;
    }

    protected void setUser(String user) {
        this.user = user;
    }

    protected void setPassword(String password) {
        this.password = password;
    }

    protected void setJmxRMIConnectorPort(int jmxRMIConnectorPort) {
        this.jmxRMIConnectorPort = jmxRMIConnectorPort;
    }

    protected void setJoramDomainPort(int joramDomainPort) {
        this.joramDomainPort = joramDomainPort;
    }

    protected void setJoramTCPPort(int joramTCPPort) {
        this.joramTCPPort = joramTCPPort;
    }

    protected void setDreamTCPPort(int dreamTCPPort) {
        this.dreamTCPPort = dreamTCPPort;
    }

}
