/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): pierre.garcia@inrialpes.fr
 */

package org.ow2.petals.transport.platform.dream;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;

import javax.jbi.JBIException;
import javax.jbi.messaging.MessageExchange;

import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageManager;
import org.objectweb.dream.message.MessageManagerType;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.topology.TopologyService;
import org.ow2.petals.jbi.messaging.endpoint.AbstractEndpoint;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl;
import org.ow2.petals.transport.ProtocolMonitor;
import org.ow2.petals.transport.TransportException;
import org.ow2.petals.transport.platform.TransportProtocol;
import org.ow2.petals.transport.platform.TransportProtocolListener;
import org.ow2.petals.transport.util.DreamUtil;
import org.ow2.petals.transport.util.TransporterUtil;
import org.ow2.petals.util.LoggingUtil;

/**
 * Dream Implementation of the Transporter
 * 
 * @author pgarcia
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "in-push", signature = Push.class),
        @Interface(name = "service", signature = TransportProtocol.class) })
public class DreamWrapper implements TransportProtocol, Push {

    // -------------------------------------------------------------------------
    // Constants
    // -------------------------------------------------------------------------

    /**
     * Name of the Dream Transporter
     */
    public static final String TRANSPORTER_NAME = "dream";

    /**
     * The default maximum size of a Blocking queue containing messageExchange.
     */
    private static final int DEFAULT_QUEUE_SIZE = 10000;

    // -------------------------------------------------------------------------
    // Fields
    // -------------------------------------------------------------------------

    /**
     * Map used to store incoming messages for poll() call
     */
    protected Map<String, Queue<MessageExchangeImpl>> incomingExchanges;

    /**
     * The maximum size of a Blocking queue containing messageExchange.
     */
    private int queueSize = DEFAULT_QUEUE_SIZE;

    // -------------------------------------------------------------------------
    // Client Interfaces
    // -------------------------------------------------------------------------

    /**
     * Interface used to get host and port of message recipient
     */
    @Requires(name = "topology", signature = org.ow2.petals.communication.topology.TopologyService.class)
    protected TopologyService topologyService;

    /**
     * Interface to send messages to another JBI environment
     */
    @Requires(name = "out-push", signature = Push.class)
    protected Push outPushItf;

    /**
     * Interface to manage dream message
     */
    @Requires(name = "message-manager", signature = MessageManager.class)
    protected MessageManagerType messageManagerItf;

    /**
     * Interface used to serialize/deserialize messages.
     */
    @Requires(name = "dispatcher")
    protected TransportProtocolListener dispatcher;

    /**
     * The Fractal component
     */
    @Service
    Component dreamTransporterComponent;

    /**
     * The fractal exposed interface of the component
     */
    TransportProtocol dreamTransportProtocol;

    // -------------------------------------------------------------------------
    // logger fields
    // -------------------------------------------------------------------------

    /**
     * Fractal Logger
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * Logger Wrapper
     */
    protected LoggingUtil log;

    /**
     * Dream Utilitity Object : used to create Chunks
     */
    protected DreamUtil dreamUtil;

    private DreamMonitoring dreamMonitoring;

    // --------------------------------------------------------------------------
    // TransportProtocol Interface Implementation
    // --------------------------------------------------------------------------

    /**
     * @see org.ow2.petals.transport.Transporter#addDestination(java.lang.String)
     */
    public void addDestination(String componentName) throws TransportException {
        log.call("Destination : " + componentName);

        if (incomingExchanges.containsKey(componentName)) {
            throw new TransportException("The destination " + componentName + " already exist");
        } else {
            incomingExchanges.put(componentName, new LinkedList<MessageExchangeImpl>());
        }

        log.end();
    }

    /**
     * @see org.ow2.petals.transport.Transporter#removeDestination(java.lang.String)
     */
    public void removeDestination(String componentName) throws TransportException {
        log.call("Destination : " + componentName);

        if (incomingExchanges.remove(componentName) == null) {
            log.warning("The destination " + componentName + " does not exist");
        }
    }

    /**
     * This method does following action :
     * <ol>
     * <li>Serialize the message exchange in a dream message.</li>
     * <li>Get the destination of the message.</li>
     * <li>Add it to the message.</li>
     * <li>Send it to the transport agent.</li>
     * </ol>
     * 
     * @see org.ow2.petals.transport.Transporter#send(org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl,
     *      java.lang.String, java.lang.String, long)
     */
    public void send(final MessageExchangeImpl messageExchange,
            final String componentName, final String containerName,
            final String subDomainName, final long timeToLive)
            throws TransportException {
        log.start();

        // clean unused messages
        messageExchange.cleanMessages();

        // Serialization of the message
        Message msg;
        try {
            // transform the JBI message into a DREAM one
            msg = dreamUtil.jbi2dream(messageExchange);
        } catch (TransportException se) {
            throw new TransportException("Error during the serialization of the message: "
                    + se.getMessage(), se);
        }
        log.debug("jbi message serialized in a dream message");

        // add destination = (host, port)
        dreamUtil.addChunkDestination(msg, containerName);
        log.debug("destination for the container " + containerName + " added in the dream message");

        // delegate the send to the transport agent
        try {
            outPushItf.push(msg);
        } catch (PushException e) {
            throw new TransportException("Error in the sending of the message \n " + e.getMessage()
                    + "with dream transporter", e);
        }

        log.end();
    }

    /**
     * @see org.ow2.petals.transport.platform.TransportProtocol#receive(java.lang.String)
     */
    public MessageExchangeImpl receive(String componentName) throws TransportException {
        log.start();

        MessageExchangeImpl responseExchange = null;

        Queue<MessageExchangeImpl> queue = this.incomingExchanges.get(componentName);
        synchronized (queue) {
            responseExchange = queue.poll();
        }

        log.end();
        return responseExchange;
    }

    // -------------------------------------------------------------------------
    // Dream Push Interface Implementation
    // -------------------------------------------------------------------------

    /**
     * Serialize the dream {@link Message message} into a
     * {@link MessageExchange jbi} one, put it in the matching queue and then
     * tell the dispatcher a new message is available.
     * 
     * @param dreamMessage
     *            the incoming dream message.
     */
    public void push(Message dreamMessage) throws PushException {
        log.start();

        // Transform the message
        MessageExchangeImpl messageExchange;
        try {
            messageExchange = dreamUtil.dream2jbi(dreamMessage);
        } catch (JBIException e) {
            throw new PushException("The message couldn't have been deserialized", e);
        }

        messageManagerItf.deleteMessage(dreamMessage);

        if (TransporterUtil.getSyncMode(messageExchange, true)) {
            log.debug("The message is a synchronized response for the component "
                    + getComponentName(messageExchange));
            dispatcher.syncMessageReceived(messageExchange);
        } else {
            log.debug("The message is not a synchronized response");
            // Store the message
            String componentName = getComponentName(messageExchange);

            Queue<MessageExchangeImpl> exchangeQueue = this.incomingExchanges.get(componentName);

            if (exchangeQueue.size() == queueSize) {
                log
                        .warning("The Dream Transporter has reached its maximum capacity for the target component '"
                                + componentName + "'. The message exchange is rejected");
            } else {
                synchronized (exchangeQueue) {
                    exchangeQueue.add(messageExchange);
                }
                // Warn the dispatcher
                dispatcher.messageAvailable(dreamTransportProtocol, componentName);
            }
        }
        
        log.end();
    }

    // --------------------------------------------------------------------------
    // Fractal Lifecycle interface implementation
    // --------------------------------------------------------------------------

    /**
     * Init variables with Fractal attributes value and create/bind the topic
     * for this container.
     * 
     * @throws NoSuchInterfaceException
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IllegalLifeCycleException, NoSuchInterfaceException {
        log = new LoggingUtil(logger);
        log.start();

        // get the exposed inteface "service"
        dreamTransportProtocol = (TransportProtocol) dreamTransporterComponent
                .getFcInterface("service");
        // create dream utility object
        dreamUtil = new DreamUtil(messageManagerItf, log, topologyService);
        // Maps initialization
        incomingExchanges = new HashMap<String, Queue<MessageExchangeImpl>>(10,0.99f);

        // Create dream monitoring instance
        dreamMonitoring = new DreamMonitoring(this);

        log.end();
    }

    /**
     * Method called when the fractal component is stopped
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() throws Exception {
        log.start();
        log.end();
    }

    // --------------------------------------------------------------------------
    // Petals Lifecycle interface implementation
    // --------------------------------------------------------------------------

    /**
     * @see org.ow2.petals.PetalsLifeCycle#shutdown()
     */
    public void shutdown() throws Exception {
        log.start();
        log.end();
    }

    // --------------------------------------------------------------------------
    // Utility methods
    // --------------------------------------------------------------------------

    /**
     * Get the matching component name
     * 
     * @param jbiMsg
     * @param componentName
     * @return
     * @throws TransportException
     */
    private String getComponentName(MessageExchangeImpl jbiMsg)

    {
        log.start();
        String componentName = null;
        if (MessageExchange.Role.CONSUMER.equals(jbiMsg.getRole())) {
            componentName = (jbiMsg.getConsumerEndpoint()).getComponentName();
            log.debug("consumer endpoint" + componentName);
        } else if (MessageExchange.Role.PROVIDER.equals(jbiMsg.getRole())) {
            componentName = ((AbstractEndpoint) jbiMsg.getEndpoint()).getComponentName();
            log.debug("service endpoint " + componentName);
        }
        log.end();
        return componentName;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.transport.platform.TransportProtocol#getProtocolMonitor()
     */
    public ProtocolMonitor getProtocolMonitor() {
        return dreamMonitoring;
    }

    /**
     * Set the size of the Queue
     * 
     * @param queueSize
     */
    public void setQueueSize(int queueSize) {
        this.queueSize = queueSize;
    }

    /**
     * Get the size of the Queue
     */
    protected int getQueueSize() {
        return queueSize;
    }
    
      /**
     * Returns a {@link Map} of component queues
     * 
     * @return a {@link Map} of component queues
     */
    protected Map<String, Queue<MessageExchangeImpl>> getComponentQueues() {
        return new HashMap<String, Queue<MessageExchangeImpl>>(incomingExchanges);
    }
}
